# Simulation results for the GLATE model
# weak-identification-robust confidence regions
# Haitian Xie

library(Rfast)

# kernel function

kern <- function(x){3/4 * (1-x^2) * (abs(x)<1)}

# DGP

dgp <- function(n,c){

X1 <- runif(n,-0.5,0.5)
X2 <- runif(n,-0.5,0.5)
X3 <- runif(n,-0.5,0.5)

Z <- as.numeric(runif(n,0,1) < (X1 + X2 + X3 )/3 + 0.5)

S <- vector(length=n)
for (i in 1:n){

  x1 <- X1[i]
  x2 <- X2[i]
  probability <- c(0.2+x1/10,
                   0.2+x2/10,
                   0.2+x1*x2/10,
                   c*abs(x1+x2)/sqrt(n),
                   0.4-(x1+x2+x1*x2)/10-c*abs(x1+x2)/sqrt(n))
  S[i] <- sample(1:5,size=1,prob = probability)
  
      }

s1 <- S == 1
s2 <- S == 2
s3 <- S == 3
s4 <- S == 4
s5 <- S == 5

T1 <- s1 + s4 * (1-Z)
T2 <- s2 + s5 * (1-Z)
T3 <- s3 + s4 * Z + s5 * Z

xi1 <- rpois(n,lambda = exp(X1+X2+X3)+1)
xi2 <- rpois(n,lambda = exp(X1+X2+X3)+2)
xi3 <- rpois(n,lambda = exp(X1+X2+X3)+3)

xi1.tilde <- rpois(n,lambda = 1)
xi2.tilde <- rpois(n,lambda = 2)
xi3.tilde <- rpois(n,lambda = 3)

Y1 <- s1 * xi1.tilde + s2 * xi2.tilde + s3 * xi3.tilde + xi1
Y2 <- s1 * xi1.tilde + s2 * xi2.tilde + s3 * xi3.tilde + xi2
Y3 <- s1 * xi1.tilde + s2 * xi2.tilde + s3 * xi3.tilde + xi3

Y <- T1 * Y1 + T2 * Y2 + T3 * Y3

return(data.frame(cbind(Y,T1,T2,T3,Z,X1,X2,X3)))

}



# Estimation

beta11_sim <- function(n,c,k,h){
  
  beta0 <- (exp(0.5)-exp(-0.5))^3

data <- dgp(n,c)

id <- sample(rep(1:k,floor(n/k)))

upsilon_vec <- vector(length = k) 
p_vec <- vector(length = k) 
psi2_vec <- vector(length=k)
  
for (fold in 1:k){
  curr <- data[id == fold,]

  others <- data[id != fold,]
  others0 <- data[id != fold & data$Z==0,]
  others1 <- data[id != fold & data$Z==1,]
  
  upsilon <- 0
  p <- 0
  psi2 <- 0
  
  for (i in 1:nrow(curr)){
    
    regressor <- cbind(rep(1,nrow(others)),
                        (others$X1 - curr$X1[i]),
                        (others$X2 - curr$X2[i]),
                        (others$X3 - curr$X3[i]))
    
    regressor0 <- cbind(rep(1,nrow(others0)),
                            (others0$X1 - curr$X1[i]),
                            (others0$X2 - curr$X2[i]),
                            (others0$X3 - curr$X3[i]))
    
    regressor1 <- cbind(rep(1,nrow(others1)),
                            (others1$X1 - curr$X1[i]),
                            (others1$X2 - curr$X2[i]),
                            (others1$X3 - curr$X3[i]))
    
    W <- kern( (curr$X1[i] - others$X1)/h ) * kern( (curr$X2[i] - others$X2)/h )  * kern( (curr$X3[i] - others$X3)/h )
    W0 <- kern( (curr$X1[i] - others0$X1)/h ) * kern( (curr$X2[i] - others0$X2)/h ) * kern( (curr$X3[i] - others0$X3)/h )
    W1 <- kern( (curr$X1[i] - others1$X1)/h ) * kern( (curr$X2[i] - others1$X2)/h ) * kern( (curr$X3[i] - others1$X3)/h ) 
    
    pi <- lmfit(regressor,others$Z,W)$be[1]
    
    Q10 <- lmfit(regressor0,others0$Y * others0$T1,W0)$be[1]
    Q11 <- lmfit(regressor1,others1$Y * others1$T1,W1)$be[1]
    
    P10 <- lmfit(regressor0,others0$T1,W0)$be[1]
    P11 <- lmfit(regressor1,others1$T1,W1)$be[1]
    
    upsilon_curr <- Q10 + (1-curr$Z[i])/(1-pi) * (curr$Y[i]*curr$T1[i] - Q10) - (Q11 + curr$Z[i]/pi * (curr$Y[i]*curr$T1[i] - Q11))
    upsilon <- upsilon + upsilon_curr
    p_curr <- P10 + (1-curr$Z[i])/(1-pi) * (curr$T1[i] - P10) - (P11 + curr$Z[i]/pi * (curr$T1[i] - P11))
    p <- p + p_curr
    psi2 <- psi2 + (upsilon_curr - beta0*p_curr)^2
  }
  
  upsilon_vec[fold] <- upsilon
  p_vec[fold] <- p
  psi2_vec[fold] <- psi2 
 
}

rho <- (sum(upsilon_vec) - beta0 * sum(p_vec))/sqrt(sum(psi2_vec))

test <- c(abs(rho)<qnorm(0.95),abs(rho)<qnorm(0.975),abs(rho)<qnorm(0.995))


return(test)

}


beta11_sim(500,0.1,2,0.5)


    for (n in c(250,500,1000)){
      for (c in 1:3){
        for (k in c(2,5)){
          for (bandwidth in c(0.7,0.8,0.9)){
            set.seed(123)
            assign(paste("result_",bandwidth,"_",k,"_",c,"_",n,sep = ""),replicate(1000,beta11_sim(n=n,c=c,k=k,h=bandwidth)))
          }
        }
      }
    }
  

rowMeans(result_0.9_5_3_250)










